/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License. You can obtain
 * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
 * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 * 
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
 * Sun designates this particular file as subject to the "Classpath" exception
 * as provided by Sun in the GPL Version 2 section of the License file that
 * accompanied this code.  If applicable, add the following below the License
 * Header, with the fields enclosed by brackets [] replaced by your own
 * identifying information: "Portions Copyrighted [year]
 * [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */
package org.glassfish.scripting.jruby.monitor;

import org.glassfish.external.probe.provider.annotations.ProbeListener;
import org.glassfish.external.probe.provider.annotations.ProbeParam;
import org.glassfish.external.statistics.CountStatistic;
import org.glassfish.external.statistics.StringStatistic;
import org.glassfish.external.statistics.impl.CountStatisticImpl;
import org.glassfish.external.statistics.impl.StringStatisticImpl;
import org.glassfish.gmbal.AMXMetadata;
import org.glassfish.gmbal.Description;
import org.glassfish.gmbal.ManagedAttribute;
import org.glassfish.gmbal.ManagedObject;

import java.util.logging.Logger;

/**
 * JRuby runtime pool stats. It is per application.
 * @author Vivek Pandey
 */
@AMXMetadata(type="jruby-runtime-pool-mon", group="monitoring")
@ManagedObject
@Description("JRuby runtime pool info")
public class JRubyRuntimePoolStatsProvider{

    private final Logger logger = Logger.getLogger(JRubyRuntimePoolStatsProvider.class.getName());

    private final StringStatisticImpl appName = new StringStatisticImpl("ApplicationName", null, "Ruby application name");
    private final StringStatisticImpl contextPath = new StringStatisticImpl("ContextPath", null, "Context path of Ruby application");

    private final CountStatisticImpl hardMinimum = new CountStatisticImpl("HardMinimum", CountStatisticImpl.UNIT_COUNT, "Minimum active runtimes");
    private final CountStatisticImpl hardMaximum = new CountStatisticImpl("HardMaximum", CountStatisticImpl.UNIT_COUNT, "Maximum active runtimes");
    private final CountStatisticImpl activeRuntimes = new CountStatisticImpl("ActiveRuntimes", CountStatisticImpl.UNIT_COUNT, "Currently active runtimes");

    public JRubyRuntimePoolStatsProvider(String appName, String contextPath) {
        this.appName.setCurrent(appName);
        this.contextPath.setCurrent(contextPath);
    }

    @ManagedAttribute(id="applicationname")
    @Description("Application name")
    public StringStatistic getApplicationName(){
        return appName;
    }


    @ManagedAttribute(id="contextpath")
    @Description("Application context-path")
    public StringStatistic getContextPath(){
        return contextPath;
    }

    @ProbeListener("glassfish:jruby:runtime-pool:runtimePoolStartEvent")
    @Description("JRuby runtime pool creation Event")
    public void runtimePoolStartedEvent(
            @ProbeParam("appName") String appName,
            @ProbeParam("hardMinimum") int hardMinimum,
            @ProbeParam("hardMaximum") int hardMaximum,
            @ProbeParam("activeRuntimes") int activeRuntimes
    ){
        if(appName == null){
            logger.warning("appName is null in runtimePoolStartedEvent! Looks like the event is not triggered properly, ignoring this monitoring event.");
            return;
        }

        this.hardMinimum.setCount(hardMinimum);
        this.hardMaximum.setCount(hardMaximum);
        this.activeRuntimes.setCount(activeRuntimes);
    }

    @ProbeListener("glassfish:jruby:runtime-pool:runtimePoolUpdateEvent")
    @Description("JRuby runtime pool update Event")
    public void runtimePoolUpdatedEvent(
            @ProbeParam("appName") String appName,
            @ProbeParam("activeRuntimes") int activeRuntimes
    ){
        if(appName == null){
            logger.warning("appName is null in runtimePoolUpdatedEvent! Looks like the event is not triggered properly, ignoring this monitoring event.");
            return;
        }

        this.activeRuntimes.setCount(activeRuntimes);
    }

    @ManagedAttribute
    public CountStatistic getHardMinimum() {
        return hardMinimum;
    }

    @ManagedAttribute
    public CountStatistic getHardMaximum() {
        return hardMaximum;
    }

    @ManagedAttribute
    public CountStatistic getActiveRuntimes() {
        return activeRuntimes;
    }
}
